PROGRAM apfel;

(*** Hiermit werden Apfelmaennchen auf dem Bildschirm dargestellt, ***)
(*** wobei der Ausschnitt der komplexen Ebene einzugeben ist!      ***)
(*** Autor: Dieter Jacob, Mai 1990                                 ***)

CONST
   xPixel = 100; (* Bildschirm: 1024 Pixel *)
   yPixel = 70;  (* Bildschirm:  780 Pixel *)
   divgrenze = 10;
   iterationsgrenze = 50;
   grossZahl = 1.0E+10;
   kleinZahl = 1.0E-05;

VAR
   xmin, xmax, ymin, ymax                   : REAL;
   xbereich, ybereich, x, y                 : REAL;
   abstandquadrat, iterationszaehler        : REAL;
   antwort                                  : CHAR;
   sichtbar                                 : BOOLEAN;
   waagPixel, senkPixel, xzaehler, yzaehler : INTEGER;

# include "graf.p";

PROCEDURE infoausgeben;
BEGIN
   WRITELN("Hiermit werden Mandelbrot-Mengen dargestellt:             ");
   WRITELN("Fuer jeden Wert c eines Bereichs der komplexen Ebene      ");
   WRITELN("wird ermittelt, ob die Iteration von a(n+1) = a(n)**2 + c ");
   WRITELN("beginnend mit a(0) = 0 divergiert oder nicht.             ");
   WRITELN("Ausgangspunkte mit Konvergenz werden weiss gezeichnet.    ");
END; (* infoausgeben *)

PROCEDURE einlesen;
BEGIN
   WRITELN("Jetzt wird der Ausschnitt der komplexen Ebene festgelegt: ");
   WRITE("Linke  Grenze (Tip: -2): "); READLN(xmin);
   WRITE("Rechte Grenze (Tip: +1): "); READLN(xmax);
   WRITE("Untere Grenze (Tip: -1): "); READLN(ymin);
   WRITE("Obere  Grenze (Tip: +1): "); READLN(ymax);
   WRITELN;
END; (* einlesen *)

PROCEDURE ausgeben;
BEGIN
   WRITELN(" ********** Mandelbrot-Menge ********** ");
   WRITELN(" Real-Bereich     : ", xmin:5:3, " bis ", xmax:5:3);
   WRITELN(" Imaginaer-Bereich: ", ymin:5:3, " bis ", ymax:5:3);
END; (* ausgeben *)

PROCEDURE Rahmen;
BEGIN
   Punkt(0,0);
   Linie_zu(0,yPixel); Linie_zu(xPixel,yPixel);
   Linie_zu(xPixel,0); Linie_zu(0,0)
END; (* Rahmen *)

PROCEDURE pruefeDivergenz(creell,cimag:REAL);
VAR zreell2, zreell, zimag : REAL;
BEGIN
   zreell := 0;
   zimag  := 0;
   iterationszaehler := 0;
   REPEAT
      iterationszaehler := iterationszaehler+1;
      (* Die Zahl z(n+1) = z(n)**2 + c wird berechnet *)
      zreell2 := zreell;
      (* zur Zwischenspeicherung! *)
      zreell:= sqr(zreell) - sqr(zimag) + creell;
      zimag := 2 * zreell2 * zimag + cimag;
      abstandquadrat := sqr(zreell) + sqr(zimag)
   UNTIL (iterationszaehler = iterationsgrenze) OR
         (abstandquadrat > grossZahl) OR (abstandquadrat < kleinZahl);
   (* Pruefung, ob der Punkt gezeichnet werden soll *)
   sichtbar := (abstandquadrat < divgrenze)
END; (*pruefeDivergenz*)

PROCEDURE transformiere;
BEGIN
   waagPixel := round(0 + (x - xmin) / xbereich * xPixel); (* 0..1023 *)
   senkPixel := round(0 + (y - ymin) / ybereich * yPixel)  (* 0..779  *)
END; (* transformiere *)

BEGIN (*** Hauptprogramm ***)
   infoausgeben;
   WRITELN;
   WHILE antwort <> "n" DO BEGIN
      einlesen;
      Tek_an;
      Loeschen;
      ausgeben;
      graf;
      Rahmen;
      xbereich := xmax - xmin;
      ybereich := ymax - ymin;
      FOR xzaehler := 0 TO xPixel DO BEGIN
         x := xmin + xbereich / xPixel * xzaehler;
         FOR yzaehler := 0 TO yPixel DO BEGIN
            y := ymin + ybereich / yPixel * yzaehler;
            pruefeDivergenz(x,y);
            IF sichtbar THEN BEGIN
               transformiere;
               Punkt(waagPixel,senkPixel)
            END (* IF *)
         END (* FOR *)
      END; (* FOR *)
   READLN;
   Tek_aus;
   WRITELN;
   WRITE("Soll das Programm wiederholt werden (j/n)? ");
   READLN(antwort);
   Loeschen
   END (*WHILE*)
END.
