PROGRAM julia;

(*** Hiermit werden Julia-Mengen auf dem Bildschirm dargestellt, ***)
(*** wobei c-Wert und Ausschnitt der Ebene eingegeben werden!    ***)
(*** Autor: Dieter Jacob, Mai 1990                               ***)

CONST
   xPixel = 200; (* Bildschirm: 1024 Pixel *)
   yPixel = 150; (* Bildschirm:  780 Pixel *)
   divgrenze = 10;
   iterationsgrenze = 50;
   grossZahl = 1.0E+10;
   kleinZahl = 1.0E-05;

VAR
   constreal, constimag, xmin, xmax, ymin, ymax: real;
   xbereich, ybereich, x, y                    : real;
   abstandquadrat, iterationszaehler           : real;
   antwort                                     : char;
   sichtbar                                    : boolean;
   waagPixel, senkPixel, xzaehler, yzaehler    : integer;

# include "graf.p";

PROCEDURE infoausgeben;
BEGIN
   WRITELN("Hiermit werden Julia-Mengen dargestellt:                  ");
   WRITELN("Fuer jeden Punkt eines Bereichs der komplexen Ebene       ");
   WRITELN("wird ermittelt, ob die Iteration von a(n+1) = a(n)**2 + c ");
   WRITELN("divergiert oder nicht.                                    ");
   WRITELN("Ausgangspunkte mit Konvergenz werden weiss gezeichnet.    ");
END; (* infoausgeben *)

PROCEDURE einlesen;
BEGIN
   WRITELN("Jetzt wird der Ausschnitt der komplexen Ebene festgelegt:");
   WRITE("Linke  Grenze: "); READLN(xmin);
   WRITE("Rechte Grenze: "); READLN(xmax);
   WRITE("Untere Grenze: "); READLN(ymin);
   WRITE("Obere  Grenze: "); READLN(ymax);
   WRITELN;
   WRITELN("Jetzt wird die Konstante c festgelegt :                  ");
        WRITE("Realteil von c     : "); READLN(constreal);
        WRITE("Imaginaerteil von c: "); READLN(constimag)
END; (* einlesen *)

PROCEDURE ausgeben;
BEGIN
   WRITELN(" ********** Julia-Menge ********** ");
   WRITELN;
   WRITELN(" Real-Bereich     : ", xmin:5:3, " bis ", xmax:5:3);
   WRITELN(" Imaginaer-Bereich: ", ymin:5:3, " bis ", ymax:5:3);
   WRITELN(" Konstante c = ", constreal:5:3, " + ",constimag:5:3, " i")
END; (* ausgeben *)

PROCEDURE Rahmen;
BEGIN
   Punkt(0,0);
   Linie_zu(0,yPixel); Linie_zu(xPixel,yPixel);
   Linie_zu(xPixel,0); Linie_zu(0,0)
END; (* Rahmen *)

PROCEDURE pruefeDivergenz(zahlreell,zahlimag:real);
var zahlreell2 : real;
BEGIN
   iterationszaehler := 0;
   REPEAT
      iterationszaehler := iterationszaehler + 1;
      (* Die Zahl (x + iy)**2 + c wird berechnet *)
      zahlreell2 := zahlreell;
      (* zur Zwischenspeicherung! *)
      zahlreell := sqr(zahlreell) - sqr(zahlimag) + constreal;
      zahlimag := 2 * zahlreell2 * zahlimag + constimag;
      abstandquadrat:=sqr(zahlreell) + sqr(zahlimag)
   UNTIL (iterationszaehler = iterationsgrenze) OR 
         (abstandquadrat > grossZahl) OR (abstandquadrat < kleinZahl);
   (* Pruefung, ob der Punkt gezeichnet werden soll *)
   sichtbar:= (abstandquadrat < divgrenze)
END; (* pruefeDivergenz *)

PROCEDURE transformiere;
BEGIN
   waagPixel := round(0 + (x - xmin) / xbereich * xPixel); (* 0..1023 *)
   senkPixel := round(0 + (y - ymin) / ybereich * yPixel)  (* 0..779  *)
END; (* transformiere *)

BEGIN (* Hauptprogramm *)
   infoausgeben;
   WRITELN;
   WHILE antwort<>"n" DO BEGIN
      einlesen;
      Tek_an;
      Loeschen;
      ausgeben;
      graf;
      Rahmen;
      xbereich := xmax - xmin;
      ybereich := ymax - ymin;
      FOR xzaehler := 0 TO xPixel DO BEGIN
         x := xmin + xbereich / xPixel * xzaehler;
         FOR yzaehler := 0 TO yPixel DO BEGIN
            y := ymin + ybereich / yPixel * yzaehler;
            pruefeDivergenz(x,y);
            IF sichtbar THEN BEGIN
               transformiere;
               Punkt(waagPixel,senkPixel)
            END (* IF *)
         END (* FOR *)
      END; (* FOR *)
      READLN;
      Tek_aus;
      WRITELN;
      WRITE("Soll das Programm wiederholt werden (j/n)? ");
      READLN(antwort);
      Loeschen
   END (*WHILE*)
END.
